#ifndef _Loublocks_h
#define _Loublocks_h

#include <windows.h>
#include <stdio.h>
#include "Shape.h"
#include "resource.h"

#define DIGITWIDTH 13
#define DIGITHEIGHT 23

#define NAMELENGTH 10 				//Maximum highscore name length

#define WIN_H 291					//Horizontal main window size
#define WIN_V 384					//Horizontal main window size

#define KEY_TIMER_ID 1000
#define DOWN_KEY_TIMER_ID 1001

bool runKeyTimer=false;

bool g_Keys[256];//need to zero memory

HWND g_hWnd = NULL;					//Global Windows handle
char g_ClassName[] = "Lou-Blocks";	//Windows class name and also window title

int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int);
LRESULT CALLBACK WndProc( HWND, UINT, WPARAM, LPARAM);

//Here are some rectangles that define the various panels for the game window
RECT TankR={12,56,172,376};		//12+(BLOCKWIDTH*10),56+(BLOCKHEIGHT*20)
RECT SideStatR={182, 11, 283, 303};
RECT NextR={182,313,283,377};		//+6(2)x,+11(1)y
RECT TopStatR={11,11,173,46};		//13+BLOCKWIDTH*10,46
RECT ScoreR={14,17,92,40};		//14+5*DIGITWIDTH,17+DIGITHEIGHT
RECT LinesR={13+ScoreR.right,17,13+ScoreR.right+3*DIGITWIDTH,17+DIGITHEIGHT};//13+ScoreR.right,17,13+ScoreR.right+3*DIGITWIDTH,17+DIGITHEIGHT
RECT LevelR={13+LinesR.right,17,13+LinesR.right+DIGITWIDTH,17+DIGITHEIGHT};

int Scores[]={10,30,50,80};	//These are the scores for 1,2,3 and 4 lines for the first level

Shape g_CurrShape;			//Global variables for shapes that will be used throughout the running application
Shape g_NextShape;
Block *g_Tank[10][20];		//The tank, a 2-D array of the blocks that make up shapes
Shape g_StatShapes[7];		//Shapes for the side statistics

struct GameStateSruct//<-name needed?		//provides important info for drawing calls and game stats
{
	BOOL Playing;		//Is game in progress(read key presses?,is timer running?)
	BOOL CurrShape;		//Is a current shape defined
	BOOL NextShape;		//Is a next shape defined
	int NextShapeType;	//what is the next shape type
	int Time;//<-------- not needed?
	int Score;			//Current score
	int Lines;			//Current number of lines
	int StartLevel;		//Set Start Level
	int Level;			//Current Level
	int Shapes[7];		//Number of times each of the seven shapes have beed created 
	void Clear()		//could ZeroMemory be used instead of all this crap?
	{
		Playing=FALSE;
		CurrShape=FALSE;
		NextShape=FALSE;
		NextShapeType=-1;
		Time=0;
		Score=0;
		Lines=0;
		Level=StartLevel;
		for (int i=0;i<7;i++)
			Shapes[i]=0;
	};
}GameState;

typedef struct		//Holds name and score for a high score
{
	int score;
	char name[NAMELENGTH]; 
}HighScore;

HighScore TopTen[10]; //Top Ten Scores, 0=top score


//Function definitions//

	//DRAWING FUNCTIONS//

void Line(HDC,int,int,int,int); 	//Draws line (Device Context,X From,Y From,X To,Y To)
void DrawBorder(HDC hdc, RECT rect, int thick, BOOL raised=FALSE);	//Draws raised or lowered border
void DrawDigits(int, int);		//Draws digits (number to draw, number of digits to draw)
void DrawBlock(HDC,Block);		//Draws a block
void DrawShape(HDC,Shape);		//Draws a shape
void DrawTank(HDC);				//Draws the entire game tank including the current shape
void DrawNextShape(HDC);		//Draws the next shape area
void DrawTopStat(HDC);			//Draws top stat information
void DrawSideStat(HDC);			//Draws side stat infomation
void Redraw(HDC);
	//SHAPE MOVMENT//

bool CheckPosition(int,int);		//Checks that a block position is valid (X,Y)
void DoMove(int);			//performs a movement (type of move)

	//HIGHSCORES//

bool Load();				//Load Data
bool Save();				//Save Data
int CheckHighScore(int score);		//Checks if high score has been achieved
BOOL CALLBACK HighScoreDlg(HWND, UINT, WPARAM, LPARAM);		//Highscore name request Dialog procedure
BOOL CALLBACK TopTenDlg(HWND, UINT, WPARAM, LPARAM);		//Top Ten Display dialog procedure
void CALLBACK KeyTimer(HWND hWnd, UINT msg, UINT idEvent, DWORD sTime);
void CALLBACK DownKeyTimer(HWND hWnd, UINT msg, UINT idEvent, DWORD sTime);

BOOL CALLBACK KeySetDlg(HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam);

void CheckKeys();

void EndGame();

void Randomise();
int Random(int highest);
void ClearTank();
void UpdateTimer();

void NewShape();
void ShapeToTank();
int CheckLines();
void RockBottom();
void UpdateScores(int lines);
void NewGame();
void ExitGame();

void ResetScores();
void FlashLines(int num, int lines[4]);
void SetLevel(int lev);

int DigitIcon[]=
{
	IDI_NUMICO_0,
	IDI_NUMICO_1,
	IDI_NUMICO_2,
	IDI_NUMICO_3,
	IDI_NUMICO_4,
	IDI_NUMICO_5,
	IDI_NUMICO_6,
	IDI_NUMICO_7,
	IDI_NUMICO_8,
	IDI_NUMICO_9,
};

int TopNamesID[]=
{
	IDC_NAME_1,
	IDC_NAME_2,
	IDC_NAME_3,
	IDC_NAME_4,
	IDC_NAME_5,
	IDC_NAME_6,
	IDC_NAME_7,
	IDC_NAME_8,
	IDC_NAME_9,
	IDC_NAME_10,
};

int TopScoresID[]=
{
	IDC_SCORE_1,
	IDC_SCORE_2,
	IDC_SCORE_3,
	IDC_SCORE_4,
	IDC_SCORE_5,
	IDC_SCORE_6,
	IDC_SCORE_7,
	IDC_SCORE_8,
	IDC_SCORE_9,
	IDC_SCORE_10,
};

int LevelID[]=
{
	ID_GAME_LEVEL_0,
	ID_GAME_LEVEL_1,
	ID_GAME_LEVEL_2,
	ID_GAME_LEVEL_3,
	ID_GAME_LEVEL_4,
	ID_GAME_LEVEL_5,
	ID_GAME_LEVEL_6,
	ID_GAME_LEVEL_7,
	ID_GAME_LEVEL_8,
	ID_GAME_LEVEL_9,
};


#endif
