#include "Shape.h"

void MoveBlock (Block *blk, int xpos, int ypos)			//moves a block to a new x and y position
{
	int xDiff=(xpos-blk->xPos)*BLOCKWIDTH;							//calc difference between new and old position
	int yDiff=(ypos-blk->yPos)*BLOCKHEIGHT;

	blk->rect.top+=yDiff;					//adjust Rect by block size multiples of the difference
	blk->rect.bottom+=yDiff;
	blk->rect.right+=xDiff;
	blk->rect.left+=xDiff;

	blk->xPos=xpos;										//set new x and y position
	blk->yPos=ypos;
}

void MoveBlock (Block *blk, RECT pRect)					//moves a block to a new RECT
{
	blk->rect.top=pRect.top+blk->yPos*BLOCKHEIGHT;
	blk->rect.left=pRect.left+blk->xPos*BLOCKWIDTH;
	
	blk->rect.bottom=blk->rect.top+BLOCKHEIGHT;
	blk->rect.right=blk->rect.left+BLOCKWIDTH;
}

void MoveBlock (Block *blk, RECT pRect, int xpos, int ypos)// moves a block to a new RECT, new x and y position
{
	MoveBlock(blk,pRect);
	MoveBlock(blk,xpos,ypos);
}

void MoveShape (Shape *shp, int xctr, int yctr)			//moves a shape to a new x and y centre position
{
	Block *blk;
	int xDist, yDist;

	for (int i=0;i<4;i++)
	{
		blk=shp->blocks[i];
		xDist=blk->xPos-shp->xCentre;					//calc distance from centre
		yDist=blk->yPos-shp->yCentre;
		MoveBlock(blk, xDist+xctr, yDist+yctr);			//adjust distance from new centre
	}

	shp->xCentre=xctr;
	shp->yCentre=yctr;
}

void MoveShape (Shape *shp, RECT pRect)					//moves a shape to a new RECT
{
	for (int i=0;i<4;i++)
		MoveBlock(shp->blocks[i],pRect);
}

void MoveShape (Shape *shp, RECT pRect, int xctr, int yctr)//moves shape to new RECT, x and y position
{
	MoveShape(shp,pRect);
	MoveShape(shp,xctr,yctr);
}

void RotateShape(Shape *shp, int type, int xoffset)
{
	int xDist, yDist, temp;

	MoveShape(shp,shp->xCentre+xoffset,shp->yCentre);

	if (type==ANTICLOCKWISE)
	{
		for (int i=0;i<4;i++)
		{
			xDist=shp->blocks[i]->xPos-shp->xCentre;
			yDist=shp->blocks[i]->yPos-shp->yCentre;

			if (shp->shiftRotate && shp->xSize%2==0)//check xsize or ysize? check whole method
				MoveBlock(shp->blocks[i],shp->xCentre+yDist,shp->yCentre-xDist+1);
			else
				MoveBlock(shp->blocks[i],shp->xCentre+yDist,shp->yCentre-xDist);
		}
			
		temp=shp->ySize;
		shp->ySize=shp->xSize;
		shp->xSize=temp;
	}

	if (type==CLOCKWISE)
	{
		for (int i=0;i<4;i++)
		{
			xDist=shp->blocks[i]->xPos-shp->xCentre;
			yDist=shp->blocks[i]->yPos-shp->yCentre;

			if (shp->shiftRotate && shp->ySize%2==0)
				MoveBlock(shp->blocks[i],shp->xCentre-yDist+1,shp->yCentre+xDist);
			else
				MoveBlock(shp->blocks[i],shp->xCentre-yDist,shp->yCentre+xDist);
		}
			
		temp=shp->ySize;
		shp->ySize=shp->xSize;
		shp->xSize=temp;
	}
}


void CreateShape(Shape *shp, int type, RECT pRect, int xctr, int yctr)
{
	COLORREF col;

	for (int i=0;i<4;i++)
	{
		shp->blocks[i]=new Block;
		shp->blocks[i]->raised=TRUE;
	}

	shp->xCentre=xctr;
	shp->yCentre=yctr;

	switch (type)
	{
		case LLAYOUT :
		{
			col=RGB_PURPLE;

			for (int i=0;i<4;i++)
				shp->blocks[i]->colour=CreateSolidBrush(col);

			MoveShape(shp,pRect);
			MoveBlock(shp->blocks[0],xctr,yctr);
			MoveBlock(shp->blocks[1],xctr+1,yctr);
			MoveBlock(shp->blocks[2],xctr-1,yctr);
			MoveBlock(shp->blocks[3],xctr-1,yctr+1);
			MoveShape(shp,pRect);
			shp->shiftRotate=FALSE;
			shp->ySize=2;
			shp->xSize=3;
		}
		break;

		case JLAYOUT :
		{
			col=RGB_RED;
			for (int i=0;i<4;i++)
				shp->blocks[i]->colour=CreateSolidBrush(col);

			MoveBlock(shp->blocks[0],xctr,yctr);
			MoveBlock(shp->blocks[1],xctr+1,yctr);
			MoveBlock(shp->blocks[2],xctr-1,yctr);
			MoveBlock(shp->blocks[3],xctr+1,yctr+1);
			MoveShape(shp,pRect);
			shp->shiftRotate=FALSE;
			shp->ySize=2;
			shp->xSize=3;
		}
		break;
		
		case SLAYOUT :
		{
			col=RGB_LIGHT_BLUE;
			for (int i=0;i<4;i++)
				shp->blocks[i]->colour=CreateSolidBrush(col);

			MoveBlock(shp->blocks[0],xctr,yctr);
			MoveBlock(shp->blocks[1],xctr+1,yctr);
			MoveBlock(shp->blocks[2],xctr,yctr+1);
			MoveBlock(shp->blocks[3],xctr-1,yctr+1);
			MoveShape(shp,pRect);
			shp->shiftRotate=TRUE;
			shp->ySize=2;
			shp->xSize=3;
		}
		break;
		
		case ZLAYOUT :
		{
			col=RGB_YELLOW;
			for (int i=0;i<4;i++)
				shp->blocks[i]->colour=CreateSolidBrush(col);

			MoveBlock(shp->blocks[0],xctr,yctr);
			MoveBlock(shp->blocks[1],xctr-1,yctr);
			MoveBlock(shp->blocks[2],xctr,yctr+1);
			MoveBlock(shp->blocks[3],xctr+1,yctr+1);
			MoveShape(shp,pRect);
			shp->shiftRotate=TRUE;
			shp->ySize=2;
			shp->xSize=3;
		}
		break;
		
		case TLAYOUT :
		{
			col=RGB_BLUE;
			for (int i=0;i<4;i++)
				shp->blocks[i]->colour=CreateSolidBrush(col);

			MoveBlock(shp->blocks[0],xctr,yctr);
			MoveBlock(shp->blocks[1],xctr+1,yctr);
			MoveBlock(shp->blocks[2],xctr-1,yctr);
			MoveBlock(shp->blocks[3],xctr,yctr+1);
			MoveShape(shp,pRect);
			shp->shiftRotate=FALSE;
			shp->ySize=2;
			shp->xSize=3;
		}
		break;
		
		case SQLAYOUT :
		{
			col=RGB_GREEN;
			for (int i=0;i<4;i++)
				shp->blocks[i]->colour=CreateSolidBrush(col);

			MoveBlock(shp->blocks[0],xctr,yctr);
			MoveBlock(shp->blocks[1],xctr+1,yctr);
			MoveBlock(shp->blocks[2],xctr,yctr+1);
			MoveBlock(shp->blocks[3],xctr+1,yctr+1);
			MoveShape(shp,pRect);
			shp->shiftRotate=TRUE;
			shp->ySize=2;
			shp->xSize=2;
		}
		break;
		
		case LINELAYOUT :
		{
			col=RGB_ORANGE;
			for (int i=0;i<4;i++)
				shp->blocks[i]->colour=CreateSolidBrush(col);

			MoveBlock(shp->blocks[0],xctr,yctr);
			MoveBlock(shp->blocks[1],xctr+1,yctr);
			MoveBlock(shp->blocks[2],xctr-1,yctr);
			MoveBlock(shp->blocks[3],xctr+2,yctr);
			MoveShape(shp,pRect);
			shp->shiftRotate=TRUE;
			shp->ySize=1;
			shp->xSize=4;
		}
		break;
	}
}

void DeleteBlock(Block *blk)
{
	DeleteObject(blk->colour); //could an if(blk) be added here to make deleting easier
	delete blk;
}

void DeleteShape(Shape *shp)
{
	for (int i=0;i<4;i++)
		if (shp->blocks[i])
		{
			DeleteBlock(shp->blocks[i]);
			shp->blocks[i]=NULL;
		}

	shp->shiftRotate=NULL;
	shp->xCentre=NULL;
	shp->yCentre=NULL;
	shp->xSize=NULL;
	shp->ySize=NULL;
}

